﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using DlhSoft.Windows.Data;

namespace GanttChartTaskListViewDatabaseSample
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        // Data context used to load, store, manage and save the local cached data from/to the database.
        private DataClassesDataContext dataContext;

        // Value used to store the project start date and time determined when loading tasks from the database.
        private DateTime projectStart = DateTime.Today;

        // Value used to store the last task index computed when loading tasks from the database and increased when new tasks are created, in order to maintain the correct order of the tasks in the database.
        private int lastTaskIndex = 0;

        public Window1()
        {
            InitializeComponent();

            // Initialize data.
            LoadData();
        }

        private void LoadMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // Clear and reinitialize data.
            ClearData();
            LoadData();
        }

        private void ClearData()
        {
            // Stop handling new item events.
            GanttChartTaskListView.TaskManager.NewPredecessorItem -= TaskManager_NewPredecessorItem;
            GanttChartTaskListView.TaskManager.NewAssignmentItem -= TaskManager_NewAssignmentItem;
            GanttChartTaskListView.TaskManager.NewAssignableResourceItem -= TaskManager_NewAssignableResourceItem;
            GanttChartTaskListView.TaskManager.NewItem -= TaskManager_NewItem;

            // Stop handling collection change events.
            ObservableCollection<Task> tasks = GanttChartTaskListView.ItemsSource as ObservableCollection<Task>;
            ObservableCollection<Resource> resources = GanttChartTaskListView.AssignableResourceItemsSource as ObservableCollection<Resource>;
            foreach (Task t in tasks)
            {
                t.Assignments.ListChanged -= Assignments_ListChanged;
                t.Predecessors.ListChanged -= Predecessors_ListChanged;
            }
            resources.CollectionChanged -= Resources_CollectionChanged;
            tasks.CollectionChanged -= Tasks_CollectionChanged;

            // Clear current data sources.
            GanttChartTaskListView.ItemsSource = null;
            GanttChartTaskListView.AssignableResourceItemsSource = null;

            // Clear internal fields.
            lastTaskIndex = 0;
            projectStart = DateTime.Today;
            dataContext.Dispose();
        }
        
        private void LoadData()
        {
            // Initialize data context.
            dataContext = new DataClassesDataContext();

            // Load tasks from the database.
            ObservableCollection<Task> tasks = new ObservableCollection<Task>(
                from t in dataContext.Tasks
                orderby t.Index
                select t);

            // Load assignable resources from the database.
            ObservableCollection<Resource> resources = new ObservableCollection<Resource>(
                from r in dataContext.Resources 
                select r);

            // The Task and related classes define the data object based on the database tables (see DataClasses.dbml).
            // The Task and related classes also define a set of supplemental properties needed in data binding processing (see DataClasses.cs).
            GanttChartTaskListView.AssignableResourceItemsSource = resources;
            GanttChartTaskListView.ItemsSource = tasks;

            // Compute and store the last task index.
            lastTaskIndex = (from t in tasks
                             orderby t.Index descending
                             select t.Index).FirstOrDefault();

            // Initialize handling collection change events.
            tasks.CollectionChanged += Tasks_CollectionChanged;
            resources.CollectionChanged += Resources_CollectionChanged;
            foreach (Task t in tasks)
            {
                t.Assignments.ListChanged += Assignments_ListChanged;
                t.Predecessors.ListChanged += Predecessors_ListChanged;
            }

            // Initialize handling new item events.
            GanttChartTaskListView.TaskManager.NewItem += TaskManager_NewItem;
            GanttChartTaskListView.TaskManager.NewAssignableResourceItem += TaskManager_NewAssignableResourceItem;
            GanttChartTaskListView.TaskManager.NewAssignmentItem += TaskManager_NewAssignmentItem;
            GanttChartTaskListView.TaskManager.NewPredecessorItem += TaskManager_NewPredecessorItem;

            // Scroll the view to properly display the project.
            DateTime overallStart = GanttChartTaskListView.OverallStart;
            DateTime overallFinish = GanttChartTaskListView.OverallFinish;
            DateTime now = DateTime.Now;
            GanttChartTaskListView.SetTimelinePage(now.AddDays(-1) <= overallStart.AddDays(-1).Add(GanttChartTaskListView.TimelinePageDuration) ? overallStart.AddDays(-1) : (now.AddDays(-1) > overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) ? overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) : now.AddDays(-1)), GanttChartTaskListView.TimelinePageDuration);
            GanttChartTaskListView.ScrollTo(now.AddDays(-1) <= overallStart.AddDays(-1).Add(GanttChartTaskListView.TimelinePageDuration) ? overallStart.AddDays(-1) : (now.AddDays(-1) > overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) ? overallFinish.AddDays(+1) : now.AddDays(-1)));
        }

        private void TaskManager_NewItem(object sender, NewItemEventArgs e)
        {
            // Creates a new task when requested.
            Task task = new Task { Index = ++lastTaskIndex, Content = "New Task", Start = projectStart >= DateTime.Today ? projectStart : DateTime.Today, Effort = TimeSpan.Parse("8:00:00") };
            // Initialize handling collection change events.
            task.Assignments.ListChanged += Assignments_ListChanged;
            task.Predecessors.ListChanged += Predecessors_ListChanged;
            // Set the new task as the new item to be used.
            e.Item = task;
        }

        private void TaskManager_NewAssignableResourceItem(object sender, NewItemEventArgs e)
        {
            // Creates a new assignable resource when requested.
            string content = (string)e.Parameters[0];
            Resource resource = new Resource { Content = content };
            // Set the new resource as the new item to be used.
            e.Item = resource;
        }

        private void TaskManager_NewAssignmentItem(object sender, NewItemEventArgs e)
        {
            // Creates a new assignment when requested.
            Resource assignmentResource = (Resource)e.Parameters[0];
            decimal allocationUnits = (decimal)(double)e.Parameters[1];
            Assignment assignment = new Assignment { ResourceItem = assignmentResource, AllocationUnits = allocationUnits };
            // Set the new assignment as the new item to be used.
            e.Item = assignment;
        }

        private void TaskManager_NewPredecessorItem(object sender, NewItemEventArgs e)
        {
            // Creates a new predecessor when requested.
            Task predecessorTask = (Task)e.Parameters[0];
            Predecessor predecessor = new Predecessor { TaskItem = predecessorTask, DependencyType = DependencyType.FinishStart, Lag = TimeSpan.Zero };
            // Set the new predecessor as the new item to be used.
            e.Item = predecessor;
        }

        private void Tasks_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // Marks adding or deleting of a task to be able to save changes later.
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (Task t in e.NewItems)
                        dataContext.Tasks.InsertOnSubmit(t);
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (Task t in e.OldItems)
                        dataContext.Tasks.DeleteOnSubmit(t);
                    break;
            }
        }

        private void Resources_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // Marks adding or deleting of a resource to be able to save changes later.
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (Resource r in e.NewItems)
                        dataContext.Resources.InsertOnSubmit(r);
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (Resource r in e.OldItems)
                        dataContext.Resources.DeleteOnSubmit(r);
                    break;
            }
        }

        private void Assignments_ListChanged(object sender, ListChangedEventArgs e)
        {
            // Marks adding or deleting of an assignment to be able to save changes later.
            BindingList<Assignment> assignments = sender as BindingList<Assignment>;
            switch (e.ListChangedType)
            {
                case ListChangedType.ItemAdded:
                    dataContext.Assignments.InsertOnSubmit(assignments[e.NewIndex]);
                    break;
                case ListChangedType.ItemChanged:
                    // The item is about to be deleted when its task relation has been removed. ListChangedType.ItemDeleted cannot be used instead because the item is inaccessible after deletion.
                    if (assignments[e.NewIndex].TaskRelation == null)
                        dataContext.Assignments.DeleteOnSubmit(assignments[e.NewIndex]);
                    break;
            }
        }

        private void Predecessors_ListChanged(object sender, ListChangedEventArgs e)
        {
            // Marks adding or deleting of a predecessor to be able to save changes later.
            BindingList<Predecessor> predecessors = sender as BindingList<Predecessor>;
            switch (e.ListChangedType)
            {
                case ListChangedType.ItemAdded:
                    dataContext.Predecessors.InsertOnSubmit(predecessors[e.NewIndex]);
                    break;
                case ListChangedType.ItemChanged:
                    // The item is about to be deleted when its task relation has been removed. ListChangedType.ItemDeleted cannot be used instead because the item is inaccessible after deletion.
                    if (predecessors[e.NewIndex].SuccessorTaskRelation == null)
                        dataContext.Predecessors.DeleteOnSubmit(predecessors[e.NewIndex]);
                    break;
            }
        }

        private void SaveMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // Save the data changes to the database.
            dataContext.SubmitChanges();
        }
    }
}
